package elokuvakatalogi;

import java.io.PrintStream;
import java.util.Random;

import fi.jyu.mit.ohj2.Mjonot;

/**
 * Elokuva-luokka tietää elokuvien tiedot, tarkistaa elokuvan syötön
 * oikeellisuuden, ja osaa muuttaa merkkijonon ..|..| elokuvan tiedoksi
 * @author Tuukka
 * @version 27.2.2025
 *
 */
public class Elokuva {
    
    private int id;
    private String nimi;
    private int vuosi;
    private String ohjaaja;
    private double imdb;
    private boolean katsottu;
    private double omaArvio;
    
    private static int seuraavaNro = 1;
    
    /**
     * Luo Elokuva-olion
     */
    public Elokuva() {
        //
    }
    
    /**
     * Tulostaa elokuvan tiedot
     * @param out Tietovirta, johon tiedot tulostetaan
     */
    public void tulosta(PrintStream out) {
        String onkoKatsottu;
        String arvioni = "";
        if(katsottu == false) {
            onkoKatsottu = "Ei katsottu";
        }
        else {
            onkoKatsottu = "Katsottu \n";
            arvioni = "Oma arvioni: " + this.omaArvio;
            // Tulostaa oman arvion vain, jos elokuva on katsottu
        }
        out.println("Nimi: " + nimi + "\n" + "Vuosi: " + vuosi + "\n" 
        + "Ohjaaja: "+  ohjaaja + "\n" + "IMDB: " + imdb
                + "\n" + onkoKatsottu + arvioni);
    }
    
    /**
     * Rekisteröi elokuvan elokuvakatalogiin ja antaa sille id:n
     * @example
     * <pre name="test">
     * Elokuva elokuva1 = new Elokuva();
     * Elokuva elokuva2 = new Elokuva();
     * elokuva1.rekisteroi(); elokuva2.rekisteroi();
     * elokuva2.getId() === elokuva1.getId() + 1;
     * </pre>
     */
    public void rekisteroi() {
        this.id = seuraavaNro;
        seuraavaNro++;
    }
    
    /**
     * Täyttää elokuvan tiedot Sawin tiedoilla
     */
    public void taytaSaw() {
        Random rand = new Random();
        this.nimi = "Saw" + rand.nextInt(100);
        this.vuosi = 2004;
        this.ohjaaja = "James Wan";
        this.imdb = 7.6;
        this.katsottu = false;
    }
    
    /**
     * @return Palauttaa elokuvan id:n
     * Elokuva elokuva1 = new Elokuva();
     * elokuva1.getId() === 0;
     * elokuva1.rekisteroi();
     * elokuva1.getId() === 1;
     */
    public int getId() {
        return this.id;
    }
    
    /**
     * Asettaa elokuvan id:n tietyksi luvuksi
     * ja päivitetään seuraavan elokuvan tuleva id
     * @param nro id, joka asetetaan elokuvalle
     */
    public void setId(int nro) {
        this.id = nro;
        if(id >= seuraavaNro) seuraavaNro = id+1;
    }

    
    /**
     * Palauttaa elokuvan nimen
     * @return elokuvan nimi
     */
    public String getNimi() {
        return this.nimi;
    }
    
    /**
     * Elokuvan tiedot eroteltuna '|'-merkillä
     */
    @Override
    public String toString() {
        return this.getId() + "|" + nimi + "|" + vuosi + "|" + ohjaaja + "|" + imdb +
                "|" + katsottu + "|" + omaArvio;
    }
    
    /**
     * Täydentää elokuvan tiedot merkkijonon perusteella
     * @param tiedot Elokuvan tiedot eroteltuna '|'-merkillä
     */
    public void parse(String tiedot) {
        var sb = new StringBuilder(tiedot);
        setId(Mjonot.erota(sb, '|', getId()));
        nimi = Mjonot.erota(sb,'|',nimi);
        vuosi = Mjonot.erota(sb, '|', vuosi);
        ohjaaja = Mjonot.erota(sb, '|', ohjaaja);
        imdb = Mjonot.erota(sb, '|', imdb);
        Mjonot.erota(sb, '|', "false");
        katsottu = sb.toString().contains("true");
        omaArvio = Mjonot.erota(sb, '|', omaArvio);
    }

}
