package elokuvakatalogi;

import java.io.PrintStream;
import java.util.Random;

import fi.jyu.mit.ohj2.Mjonot;

/**
 * - Pitää yllä rekisteriä näyttelijöistä eli osaa
 *   lisätä ja poistaa näyttelijän
 * - Lukee ja kirjoittaa näyttelijät tiedostoon
 * - Osaa etsiä ja lajitella
 * @author Tuukka
 * @version 17.3.2025
 *
 */
public class Nayttelija {
    
    private int id;
    private int tunnusNro;
    private String nimi = "";
    
    private static int seuraavaNro = 1;
    
    /**
     * Näyttelijä-luokan konstruktori ilman parametreja
     */
    public Nayttelija() {
        //
    }
    
    /** Muodostaa Näyttelijä-olion, jolle annetaan tietty id
     * @param elokuvaId id, joka annetaan näyttelijälle
     */
    public Nayttelija(int elokuvaId) {
        this.id = elokuvaId;
    }
    
    /**
     * @return näyttelijän id:n, joka kytkee hänet johonkin elokuvaan
     */
    public int getId() {
        return this.id;
    }
    
    /**
     * @return näyttelijän nimen
     * comtest
     */
    public String getNimi() {
        return this.nimi;
    }
    
    /**
     * Antaa näyttelijän nimeksi Willem Dafoen ja satunnaisen
     * kokonaisluvun väliltä 0-1000
     * @param elokuvaId ID, jolla näyttelijä yhdistetään elokuvaan
     */
    public void taytaWillemDafoe(int elokuvaId) {
        this.id = elokuvaId;
        Random rand = new Random();
        this.nimi = "Willem Dafoe" + rand.nextInt(1000);
    }
    
    /**
     * Rekisteröi näyttelijän elokuvakatalogiin ja antaa sille tunnusnumeron
     * @example
     * <pre name="test">
     * Nayttelija nayt1 = new Nayttelija(); Nayttelija nayt2 = new Nayttelija();
     * nayt1.rekisteroi();
     * nayt2.rekisteroi();
     * </pre>
     */
    public void rekisteroi() {
        this.tunnusNro = seuraavaNro;
        seuraavaNro++;
    }
    
    /**
     * @return Näyttelijän tunnusnumeron
     */
    public int getTunnusNro() {
        return this.tunnusNro;
    }
    
    /**
     * Asettaa näyttelijälle halutun id:n
     * @param nro mikä id näyttelijälle asetetaan
     */
    public void setTunnusNro(int nro) {
        id = nro;
        if(id >= seuraavaNro) seuraavaNro = id + 1;
    }


    /**
     * Tulostaa näyttelijän nimen
     * @param os Tietovirto, johon tulostetaan
     */
    public void tulosta(PrintStream os) {
        os.println(this.nimi);
    }
    
    /**
     * Näyttelijän tiedot eroteltuna '|'-merkillä
     */
    @Override
    public String toString() {
        return id + "|" + nimi;
    }
    
    /**
     * Täyttää näyttelijän tiedot merkkijonon perusteella
     * @param tiedot Näyttelijän tiedot
     */
    public void parse(String tiedot) {
        var sb = new StringBuilder(tiedot);
        setTunnusNro(Mjonot.erota(sb, '|', getTunnusNro()));
        nimi = Mjonot.erota(sb,'|',nimi);
    }
}
