package fxtyo2;

import elokuvakatalogi.Elokuvakatalogi;
import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.ModalController;
import fi.jyu.mit.fxgui.ModalControllerInterface;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.TextField;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;
import javafx.scene.layout.Pane;
import javafx.stage.Stage;

/** Kysytään käyttäjältä käyttäjätunnusta ja kirjataan hänet sisään
 * 
 * @author Tuukka
 * @version 19.2.2025
 *
 */
public class KayttoliittymaGUIController {
    @FXML
    private TextField kayttajaNimi;
    private String kayttajaSyote = null;

    /*
     * Yritetään avata pääikkuna, kun henkilö syöttää käyttäjänimen
     * ENTER-napilla
     */
    @FXML
    void handleKirjautuminenEnter(KeyEvent nappi) {
        if(nappi.getCode() == KeyCode.ENTER) {
            handleKirjautuminen();
        }
    }

    /*
     * Yritetään kirjata henkilö sisään, kun hän painaa 
     * "Kirjaudu sisään"-nappia.
     */
    @FXML
    private void handleKirjautuminen() {
        kayttajaSyote = kayttajaNimi.getText(); // Otetaan henkilön syöte
        Dialogs.showMessageDialog("Vielä ei osata kirjautua sisään");
        // Sisäänkirjautuminen ei vielä virallisesti toimi
        
        avaaPaavalikko(kayttajaSyote); // Avataan päävalikko
    }

    /**
     * Ladataan ohjelman pääikkuna, kun käyttäjä antaa oikean käyttäjänimen
     * tai tekee uuden elokuvakatalogin
     * @param valikkoNimi elokuvakatalogin nimi. Asetetaan ohjelman nimeksi
     */
    private void avaaPaavalikko(String valikkoNimi) {
        try {
            final FXMLLoader ldr = new FXMLLoader(getClass().getResource("paaikkunaGUIView.fxml"));
            final Pane root = (Pane)ldr.load();
            final paaikkunaGUIController paaikkunaControl = (paaikkunaGUIController)ldr.getController(); 
            paaikkunaControl.setNimi(valikkoNimi);
            Scene scene = new Scene(root);
            scene.getStylesheets().add(getClass().getResource("kayttoliittyma.css").toExternalForm());
            Stage paaValikko = new Stage();
            paaValikko.setScene(scene);
            paaValikko.setTitle("Elokuvakatalogi - " + valikkoNimi);
            
            paaValikko.show();
            // Näytetään ohjelman pääikkuna
            
            kayttajaNimi.getScene().getWindow().hide();
            // Piilotetaan kirjautumisikkuna
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    /*
     * Luodaan uusi elokuvakatalogi
     */
    @FXML
    private void handleUusiKayttaja() {
        avaaPaavalikko("Uusi käyttäjä");
        // Ei vielä toiminnallisuutta
    }
    
    private Elokuvakatalogi elokuvakatalogi;

    /**
     * Asetetaan käytettävä elokuvakatalogi
     * @param elokuvakatalogi jota käytetään
     */
    public void setElokuvakatalogi(Elokuvakatalogi elokuvakatalogi) {
        this.elokuvakatalogi = elokuvakatalogi;
    }
}