package fxtyo2;

import java.io.PrintStream;
import java.net.URL;
import java.util.ResourceBundle;

import elokuvakatalogi.Elokuva;
import elokuvakatalogi.Elokuvakatalogi;
import elokuvakatalogi.Nayttelija;
import elokuvakatalogi.SailoException;
import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.ListChooser;
import fi.jyu.mit.fxgui.ModalController;
import fi.jyu.mit.fxgui.TextAreaOutputStream;
import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Button;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.MenuItem;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.TextArea;
import javafx.scene.text.Font;

/**
 * Elokuvakatalogin pääikkuna, jossa on ohjelman päätoiminnot kuten
 * elokuvan lisääminen ja muokkaaminen yms.
 * @author Tuukka
 * @version 17.2.2025
 *
 */
public class paaikkunaGUIController implements Initializable{
    @FXML
    private MenuItem apuMenuNappi;
    
    @FXML
    private ScrollPane elokuvaScroll;

    @FXML
    private MenuItem avaaMenuNappi;

    @FXML
    private ChoiceBox<String> choiceIMDB;

    @FXML
    private ChoiceBox<String> choiceKatsotut;

    @FXML
    private ChoiceBox<String> choiceOmaArvio;

    @FXML
    private ChoiceBox<String> choiceVuosi;

    @FXML
    private MenuItem katsotuksiMenuNappi;

    @FXML
    private MenuItem poistaElokuvaMenuNappi;
    
    @FXML
    private MenuItem muokkaaElokuvaaMenuNapppi;
    
    @FXML
    private MenuItem uusiNayttelijaNappi;

    @FXML
    private MenuItem suljeMenuNappi;

    @FXML
    private MenuItem tallennaMenuNappi;

    @FXML
    private MenuItem uusiElokuvaMenuNappi;

    @FXML
    private Button uusiElokuvaNappi;
    
    @FXML
    private Button tallennaNappi;
    
    @FXML
    private ListChooser<Elokuva> chooserElokuvat1;
    
    @FXML
    private ListChooser<Elokuva> chooserElokuvat2;
    
    private String[] numeerisetValinnat = {">=","<="};
    private String[] katsomisValinnat = {"Kaikki","Katsotut","Katsomattomat"};
    
    private Elokuvakatalogi elokuvakatalogi = new Elokuvakatalogi();
    
    private TextArea areaElokuva = new TextArea();
    
    private Elokuva elokuvaKlikattu;
    
    String nimi;
    
    @Override
    public void initialize(URL arg0, ResourceBundle arg1) {
        alusta();
        lueTiedostosta("data");
        choiceIMDB.getItems().addAll(numeerisetValinnat);
        choiceOmaArvio.getItems().addAll(numeerisetValinnat);
        choiceVuosi.getItems().addAll(numeerisetValinnat);
        // Annetaan numeerisilla valintalaatikoille valinnat >= ja <=
        
        choiceKatsotut.getItems().addAll(katsomisValinnat);
        // Elokuvia voidaan valikoida katsomisen mukaan
        
        
    }
    
    private void lueTiedostosta(String hakemisto) {
        // String elokuvaNimi = nimi;
        // String nayttelijaNimi = nimi;
        
        try {
            elokuvakatalogi.lueTiedostosta(hakemisto);
            hae(0);
        } catch (SailoException e) {
            Dialogs.showMessageDialog(e.getMessage());
        }
        
    }

    /*
     * Avaa käyttäjälle apusivun, jossa on tietoa ohjelmasta
     */
    @FXML
    private void handleApusivu() {
        Dialogs.showMessageDialog("Apusivua ei ole tehty vielä");
        // Ei vielä toiminnallisuutta
    }
    
    /*
     * Merkitään elokuva katsotuksi ja annetaan käyttäjälle mahdollisuus
     * arvioida se
     */
    @FXML
    private void handleMerkitseKatsotuksi() {
        ModalController.showModal(paaikkunaGUIController.class.getResource("merkitseKatsotuksiGUIView.fxml"), "Merkitse katsotuksi", null, "");
    }
    
    /*
     * Avataan tietty elokuvakatalogi
     */
    @FXML
    private void handleAvaaTiedosto() {
        Dialogs.showMessageDialog("Emme osaa avata tiedostoja vielä :(");
        // Ei toiminnallisuutta vielä
    }
    
    /*
     * Ohjataan sivulle, jossa voidaan lisätä uusi elokuva
     */
    @FXML
    private void handleUusiElokuva() {
        lisaaElokuvaSivu();
    }
    
    @FXML
    private void handleUusiNayttelija() {
        uusiNayttelija();
    }
    
    private void alusta() {
        chooserElokuvat1.clear();
        chooserElokuvat2.clear();
        
        elokuvaScroll.setContent(areaElokuva);
        areaElokuva.setFont(new Font("Comic Sans MS", 10));
        elokuvaScroll.setFitToHeight(true);
        
        chooserElokuvat1.addSelectionListener(_ -> naytaElokuva(chooserElokuvat1));
        chooserElokuvat2.addSelectionListener(_ -> naytaElokuva(chooserElokuvat2));

    }

    /**
     * Näyttää klikatun elokuvan tiedot
     * @param chooser kumpaan valitsijaan elokuva kuuluu
     */
    private void naytaElokuva(ListChooser<Elokuva> chooser) {
        if(chooser == chooserElokuvat1) chooserElokuvat2.setSelectedIndex(-1);
        if(chooser == chooserElokuvat2) chooserElokuvat1.setSelectedIndex(-1);
        // Jotta toisen chooserin valinta ei jäisi päälle
        elokuvaKlikattu = chooser.getSelectedObject();
        if(elokuvaKlikattu == null) return;
        
        areaElokuva.setText("");
        try (PrintStream os = TextAreaOutputStream.getTextPrintStream(areaElokuva)) {
            tulosta(os,elokuvaKlikattu);  
        }
    }

    private void uusiNayttelija() {
        if(elokuvaKlikattu == null) return;
        Nayttelija uusiNayt = new Nayttelija(elokuvaKlikattu.getId());
        uusiNayt.rekisteroi();
        uusiNayt.taytaWillemDafoe(elokuvaKlikattu.getId());
        elokuvakatalogi.lisaa(uusiNayt);
        hae(elokuvaKlikattu.getId());
    }

    private void tulosta(PrintStream os, Elokuva elokuva) {
        elokuvakatalogi.tulosta(os,elokuva);  
    }


    /**
     * Avaa sivun elokuvan lisäämistä varten
     */
    private void lisaaElokuvaSivu() {
        Elokuva uusi = new Elokuva();
        uusi.rekisteroi();
        uusi.taytaSaw();
        try {
            elokuvakatalogi.lisaa(uusi);
        } catch (SailoException e) {
            Dialogs.showMessageDialog("Elokuvaa ei voitu lisätä " + e.getMessage());
            return;
        }
        
        hae(uusi.getId());
        ModalController.showModal(lisaaelokuvaGUIController.class.getResource("lisaaelokuvaGUIView.fxml"), "Uusi elokuva", null, "");
    }

    private void hae(int id) {
        chooserElokuvat1.clear();
        chooserElokuvat2.clear();
        
        int indeksi = 0;
        for (int i = 0; i < elokuvakatalogi.getElokuvia(); i+=2) {
            Elokuva elokuva = elokuvakatalogi.annaElokuva(i);
            if (elokuva.getId() == id) {
                indeksi = i; 
                elokuvaKlikattu = elokuva;
            }
            chooserElokuvat1.add(elokuva.getNimi(), elokuva);
            if (i+1 < elokuvakatalogi.getElokuvia()) {
                elokuva = elokuvakatalogi.annaElokuva(i+1);
                chooserElokuvat2.add(elokuva.getNimi(),elokuva);
            if (elokuva.getId() == id) {
                indeksi = i+1; 
                elokuvaKlikattu = elokuva;
            }
            }
        }
        if(indeksi == 0) chooserElokuvat1.setSelectedIndex(indeksi);
        if (indeksi % 2 == 0) {
            chooserElokuvat1.setSelectedIndex(indeksi/2);
        }
        else {
            chooserElokuvat2.setSelectedIndex((indeksi-1)/2); 
        }
        // Tarpeelliset muutokset, koska listChoosereita on kaksi ja 
        // indeksit menisivät muuten väärin
    }
    
    /*
     * Poistetaan jokin tietty elokuva tietokannasta
     */
    @FXML
    private void handlePoistaElokuva() {
        Dialogs.showMessageDialog("Emme osaa poistaa elokuvia vielä :(");
        // Ei toimi vielä
    }
    
    /*
     * Tallennetaan tiedostoon tehdyt muutokset
     */
    @FXML
    private void handleTallennaTiedosto() throws SailoException{
        tallennaTiedosto();
    }
    
    /**
     * Tallentaa elokuvien tiedot tiedostoon
     * @throws SailoException mikäli tiedostoa ei voida tallentaa
     */
    public void tallennaTiedosto() throws SailoException {      
        elokuvakatalogi.tallenna("data");
    }

    /*
     * Suljetaan tiedosto. Tulevaisuudessa tallennetaan tehdyt muutokset
     */
    @FXML
    private void handleSuljeTiedosto() {
        Platform.exit();
    }
    
    /*
     * Avaa ikkunan, jossa voidaan muokata elokuvan tietoja
     */
    @FXML
    private void handleMuokkaaElokuvaa() {
        ModalController.showModal(muokkaaElokuvaaGUIController.class.getResource("muokkaaElokuvaaGUIView.fxml"), "Muokkaa elokuvaa", null, "");
    }
    
    /**
     * @param valikkoNimi mikä nimi elokuvakatalogille asetetaan
     */
    public void setNimi(String valikkoNimi) {
        nimi = valikkoNimi;
    }
}
